# @Coding: UTF-8
# @Time: 2024/9/24 12:58
# @Author: xieyang_ls
# @Filename: annotation.py

def Get(path: str):
    if not isinstance(path, str):
        raise ValueError('GET Method: path should be a string')
    if len(path) == 0:
        raise ValueError('GET Method: path should not be empty')

    def decorator_get_func(func):
        func.__decorator__ = "GET"
        func.__decorator_path__ = path
        return func

    return decorator_get_func


def Post(path: str):
    if not isinstance(path, str):
        raise ValueError('POST Method: path should be a string')
    if len(path) == 0:
        raise ValueError('POST Method: path should not be empty')

    def decorator_post_func(func):
        func.__decorator__ = "POST"
        func.__decorator_path__ = path
        return func

    return decorator_post_func


def Put(path: str):
    if not isinstance(path, str):
        raise ValueError('PUT Method: path should be a string')
    if len(path) == 0:
        raise ValueError('PUT Method: path should not be empty')

    def decorator_put_func(func):
        func.__decorator__ = "PUT"
        func.__decorator_path__ = path
        return func

    return decorator_put_func


def Delete(path: str):
    if not isinstance(path, str):
        raise ValueError('DELETE Method: path should be a string')
    if len(path) == 0:
        raise ValueError('DELETE Method: path should be empty')

    def decorator_delete_func(func):
        func.__decorator__ = "DELETE"
        func.__decorator_path__ = path
        return func

    return decorator_delete_func



# @Coding: UTF-8
# @Time: 2024/9/11 16:10
# @Author: xieyang_ls
# @Filename: python_spark_handler.py

import os

from pyutils_spirit.exception import ArgumentException

from typing import Any, Callable, TypeVar

from logging import info, INFO, basicConfig

from pyspark import SparkConf, SparkContext, RDD

basicConfig(level=INFO)

T = TypeVar('T')

U = TypeVar('U')


class PySparkHandler:
    def __init__(self, python_environ_path: str) -> None:
        os.environ["PYSPARK_PYTHON"] = python_environ_path
        conf = SparkConf().setMaster("local[*]").setAppName("test_spark_app")
        sc = SparkContext(conf=conf)
        self.sparkContext = sc
        info(f"Spark version: {sc.version}, Spark created successfully!!!")

    def acceptData(self, data: [list, tuple, dict, set, str]) -> RDD[Any]:
        rdd = self.sparkContext.parallelize(data)
        return rdd

    def acceptFile(self, filePath: str) -> RDD[Any]:
        rdd = self.sparkContext.textFile(filePath)
        return rdd

    def operateBySelfAlgorithm(self, data: [list, tuple, dict, set, str],
                               func: Callable[[T], U], filePath: str = "") -> RDD[Any]:
        if len(data) == 0 and filePath == "":
            raise ArgumentException("please check operate callable arguments")
        if func is None:
            raise ArgumentException("Type Callable of Func argument must be not None")
        if len(data) == 0:
            rdd = self.sparkContext.textFile(filePath).map(func)
        else:
            rdd = self.sparkContext.parallelize(data).map(func)
        return rdd

    def destroy(self) -> None:
        self.sparkContext.stop()




# @Coding: UTF-8
# @Time: 2024/9/20 12:20
# @Author: xieyang_ls
# @Filename: __init__.py.py

from pyutils_spirit.tcp.websocket import WebSocketServer, Session, onopen, onmessage, onclose, onerror

__all__ = ['WebSocketServer',
           'Session',
           'onopen',
           'onmessage',
           'onclose',
           'onerror']



# @Coding: UTF-8
# @Time: 2024/9/22 17:15
# @Author: xieyang_ls
# @Filename: spirit_application.py
import os

import cgi

import json

import inspect

import importlib.util

from threading import Lock

from typing import Callable

from types import FunctionType, MethodType

from pyutils_spirit.style import draw_spirit_banner

from logging import info, INFO, basicConfig, exception

from http.server import BaseHTTPRequestHandler, HTTPServer

from pyutils_spirit.spirit_container.request_result import Result

from pyutils_spirit.util import Assemble, HashAssemble, Set, HashSet

from pyutils_spirit.spirit_container.multipart_file import MultipartFile

from pyutils_spirit.exception import NoneSignatureError, BusinessException, SpiritContainerServiceException

basicConfig(level=INFO)


class SpiritApplication:
    start_status: bool = None

    __unified_response_type__: bool = None

    __unique_modules: Set[object] = None

    __current_file__: str = None

    __current_work_dir__: str = None

    __signatures: tuple[str] = None

    __container: Assemble[str, object] = None

    __auto_wired_set__: set[object] = None

    __method_types: tuple[str] = None

    __interrupt_handlers: tuple[str] = None

    __lock: Lock = None

    __controller_paths_set: set[str] = None

    __interceptor_paths_set: set[str] = None

    __interceptor_function_before__: Callable[[BaseHTTPRequestHandler], tuple[int, bool]] = None

    __interceptor_function_after__: Callable[[BaseHTTPRequestHandler], None] = None

    def __init__(self, __file__):
        SpiritApplication.__lock = Lock()
        self.__unique_modules = HashSet()
        self.__current_file__ = os.path.basename(__file__)
        SpiritApplication.__signatures = ("Component", "Mapper", "Service", "Controller", "WebSocketServer")
        SpiritApplication.__container = HashAssemble()
        self.__auto_wired_set__ = set()
        SpiritApplication.__method_types = ("GET", "POST", "PUT", "DELETE")
        SpiritApplication.__controller_paths_set = set()
        SpiritApplication.__interceptor_paths_set = set()

    def start(self, host: str, port: int, unified_response_type: bool = True):
        SpiritApplication.__lock.acquire()
        try:
            if SpiritApplication.start_status is True:
                raise SpiritContainerServiceException
            SpiritApplication.start_status = True
        finally:
            SpiritApplication.__lock.release()
        SpiritApplication.__unified_response_type__ = unified_response_type
        draw_spirit_banner()
        self.__current_work_dir__ = os.getcwd()
        self.__scan_modules(self.__current_work_dir__)
        self.__start_service(host=host, port=port)

    def __scan_modules(self, work_directory: str):
        for dirpath, dirnames, filenames in os.walk(work_directory):
            for file_name in filenames:
                if file_name == "__init__.py":
                    continue
                if file_name.endswith('.py'):
                    file_path = os.path.join(dirpath, file_name)
                    self.__load_module(file_path)
        self.__auto_injected()

    def __load_module(self, file_path):
        current_file_path = os.path.abspath(self.__current_file__)
        if file_path == current_file_path:
            return None
        module_path = file_path[:-3]
        module_name = os.path.basename(module_path)
        spec = importlib.util.spec_from_file_location(module_name, file_path)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        self.__analyze_module(module)

    def __analyze_module(self, module) -> None:
        for name, obj in inspect.getmembers(module):
            if self.__unique_modules.__contains__(obj) is False:
                self.__unique_modules.add(obj)
                if obj is None:
                    continue
                if isinstance(obj, FunctionType | type):
                    decorator = getattr(obj, "__decorator__", None)
                    if decorator in SpiritApplication.__signatures:
                        self.__auto_wired_set__.add(obj())
                        if decorator == "Controller":
                            params = getattr(obj, "__decorator_params__")
                            SpiritApplication.__controller_paths_set.add(params)

    def __auto_injected(self) -> None:
        for instance in self.__auto_wired_set__:
            for method_name, method in inspect.getmembers(instance):
                if isinstance(method, MethodType):
                    decorator = getattr(method, "__decorator__", None)
                    if decorator == "Resource":
                        if hasattr(method, "__self__") and isinstance(method.__self__, type):
                            method(cls=type(instance), resources={})
                        else:
                            method(self=instance, resources={})

    def __start_service(self, host: str, port: int):
        server_address: tuple[str, int] = (host, port)
        service = HTTPServer(server_address, self.RequestHandler)
        info("Spirit Container Service Startup successfully")
        info(f"Listening on Server: {host}:{port}")
        service.serve_forever()

    @classmethod
    def set_interceptor_paths(cls, interceptor_paths: set[str]) -> None:
        cls.__interceptor_paths_set = interceptor_paths

    @classmethod
    def set_interceptor_before_function(cls,
                                        interceptor_before_function:
                                        Callable[[BaseHTTPRequestHandler], tuple[int, bool]]) -> None:
        cls.__interceptor_function_before__ = interceptor_before_function

    @classmethod
    def set_interceptor_after_function(cls,
                                       interceptor_after_function:
                                       Callable[[BaseHTTPRequestHandler], None]) -> None:
        cls.__interceptor_function_after__ = interceptor_after_function

    @classmethod
    def __do_interceptor_function__(cls, request_path: str) -> bool:
        if len(cls.__interceptor_paths_set) == 0:
            return False
        for path in cls.__interceptor_paths_set:
            path_list = request_path.split(path, maxsplit=1)
            if len(path_list) == 1:
                return False
            if path_list[0] == "" and path_list[1][0] == "?":
                return True
        return False

    @classmethod
    def get_func_kwargs(cls, path: str, method_type: str) -> [object, callable, dict]:
        cls.__lock.acquire()
        try:
            for controller_path in cls.__controller_paths_set:
                path_list = path.split(controller_path, maxsplit=1)
                if len(path_list) == 1:
                    continue
                if path_list[0] != "":
                    continue
                controller_func_path = path_list[1]
                if len(path_list) == 2 and controller_func_path[0] == "/":
                    controller = cls.__container.get(controller_path)
                    for method_name, method in inspect.getmembers(controller):
                        decorator = getattr(method, "__decorator__", None)
                        if decorator == method_type:
                            func_path = getattr(method, "__decorator_path__")
                            func_path_list = controller_func_path.split(func_path, maxsplit=1)
                            if len(func_path_list) == 1:
                                continue
                            func_args = func_path_list[1]
                            if len(func_path_list) > 1:
                                if func_args == "":
                                    return method, None
                                elif func_args[0] == "?":
                                    kwargs = list()
                                    for param in func_args.split("?")[1].split("&"):
                                        if "=" in param:
                                            value = param.split("=")[1]
                                            kwargs.append(value)
                                    return method, kwargs
        finally:
            cls.__lock.release()
        raise ValueError(f"please check the path {path} of {method_type} Method")

    class RequestHandler(BaseHTTPRequestHandler):

        __data__ = None

        __response__ = None

        __is_interceptor__: bool = False

        __is_pass__: bool = False

        __response_code__: int = 200

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)

        def __request_exception_advice__(self, e: Exception):
            exception(e)
            self.__response_code__ = 500
            self.__response__ = str(e)
            if isinstance(e, BusinessException):
                if SpiritApplication.__unified_response_type__ is True:
                    self.__response__ = Result.WARN(self.__response__)
            else:
                if SpiritApplication.__unified_response_type__ is True:
                    self.__response__ = Result.ERROR(self.__response__)

        def __is_interceptor_func__(self):
            self.__is_interceptor__ = SpiritApplication.__do_interceptor_function__(self.path)
            if self.__is_interceptor__:
                if SpiritApplication.__interceptor_function_before__ is not None:
                    self.__response_code__, self.__is_pass__ = SpiritApplication.__interceptor_function_before__(self)
                    if self.__is_pass__ is False:
                        self.__response__ = Result.WARN(f"the request '{self.path}' is be intercepted")
                        return True
            return False

        def __request_end_func__(self):
            self.send_response(self.__response_code__)
            self.send_header('Access-Control-Allow-Origin', '*')
            self.send_header('Content-type', 'application/json')
            self.end_headers()
            self.wfile.write(json.dumps(self.__response__).encode('utf-8'))
            if self.__is_interceptor__ and self.__is_pass__:
                if SpiritApplication.__interceptor_function_after__ is not None:
                    SpiritApplication.__interceptor_function_after__(self)

        def do_OPTIONS(self):
            self.send_response(200)
            self.send_header('Access-Control-Allow-Origin', '*')
            self.send_header('Access-Control-Allow-Methods', 'GET, POST, PUT, DELETE')
            self.send_header('Access-Control-Allow-Headers', 'Content-Type')
            self.end_headers()

        def do_GET(self):
            try:
                if self.__is_interceptor_func__():
                    return None
                func, kwargs = SpiritApplication.get_func_kwargs(self.path, "GET")
                if isinstance(kwargs, list):
                    self.__response__ = func(*kwargs)
                else:
                    self.__response__ = func()
                if SpiritApplication.__unified_response_type__ is True:
                    self.__response__ = Result.SUCCESS(self.__response__)
            except Exception as e:
                self.__request_exception_advice__(e)
            finally:
                self.__request_end_func__()

        def __get_post_file__(self, post_body_length):
            if self.headers['Content-Type'].startswith('multipart/form-data'):
                # 解析请求体
                form = cgi.FieldStorage(fp=self.rfile,
                                        headers=self.headers,
                                        environ={'REQUEST_METHOD': 'POST'})
                # 处理文件
                if "file" in form:
                    temp_file_handler = form["file"]
                elif "data" in form:
                    temp_file_handler = form["data"]
                else:
                    self.send_response(400)
                    self.end_headers()
                    raise FileNotFoundError("Post Method: Content-Type is multipart/form-data But No file uploaded")
                # 创建临时文件并写入上传的内容
                file = MultipartFile(temp_file_handler=temp_file_handler.file)
                return file
            else:
                data = self.rfile.read(post_body_length)
                return json.loads(data)

        def do_POST(self):
            post_body_length = int(self.headers['Content-Length'])
            try:
                if self.__is_interceptor_func__():
                    return None
                func, kwargs = SpiritApplication.get_func_kwargs(self.path, "POST")
                if post_body_length > 0:
                    self.__data__ = self.__get_post_file__(post_body_length)
                else:
                    self.__response_code__ = 400
                    raise ValueError("Post Method: Request Body must be not empty.")
                if isinstance(kwargs, list):
                    self.__response__ = func(*kwargs, self.__data__)
                else:
                    self.__response__ = func(self.__data__)
                if SpiritApplication.__unified_response_type__ is True:
                    self.__response__ = Result.SUCCESS(self.__response__)
            except Exception as e:
                self.__request_exception_advice__(e)
            finally:
                if isinstance(self.__data__, MultipartFile):
                    self.__data__.close()
                self.__request_end_func__()

        def do_PUT(self):
            put_body_length = int(self.headers['Content-Length'])
            if put_body_length > 0:
                data = self.rfile.read(put_body_length)
                self.__data__ = json.loads(data)
            else:
                self.__data__ = None
            try:
                if self.__is_interceptor_func__():
                    return None
                func, kwargs = SpiritApplication.get_func_kwargs(self.path, "PUT")
                if isinstance(kwargs, list):
                    if self.__data__ is None:
                        self.__response__ = func(*kwargs)
                    else:
                        self.__response__ = func(*kwargs, self.__data__)
                else:
                    if self.__data__ is None:
                        self.__response__ = func()
                    else:
                        self.__response__ = func(self.__data__)
                if SpiritApplication.__unified_response_type__ is True:
                    self.__response__ = Result.SUCCESS(self.__response__)
            except Exception as e:
                self.__request_exception_advice__(e)
            finally:
                self.__request_end_func__()

        def do_DELETE(self):
            content = self.headers['Content-Length']
            if content is not None:
                delete_body_length = int(content)
                if delete_body_length > 0:
                    data = self.rfile.read(delete_body_length)
                    self.__data__ = json.loads(data)
                else:
                    self.__data__ = None
            else:
                self.__data__ = None
            try:
                if self.__is_interceptor_func__():
                    return None
                func, kwargs = SpiritApplication.get_func_kwargs(self.path, "DELETE")
                if isinstance(kwargs, list):
                    if self.__data__ is None:
                        self.__response__ = func(*kwargs)
                    else:
                        self.__response__ = func(*kwargs, self.__data__)
                else:
                    if self.__data__ is None:
                        self.__response__ = func()
                    else:
                        self.__response__ = func(self.__data__)
                if SpiritApplication.__unified_response_type__ is True:
                    self.__response__ = Result.SUCCESS(self.__response__)
            except Exception as e:
                self.__request_exception_advice__(e)
            finally:
                self.__request_end_func__()

    @classmethod
    def Component(cls, signature: str) -> callable:
        if not isinstance(signature, str):
            raise NoneSignatureError
        if len(signature) == 0:
            raise ValueError("Component: Signature cannot be empty")

        def get_component_cls(other_cls):

            def get_component_instance(*args, **kwargs) -> object:
                instance = cls.__container.get(signature)
                if instance is None:
                    instance = other_cls(*args, **kwargs)
                    cls.__container.put(signature, instance)
                return instance

            get_component_instance.__decorator__ = "Component"
            get_component_instance.__decorator_params__ = signature
            return get_component_instance

        return get_component_cls

    @classmethod
    def Mapper(cls, signature: str) -> callable:
        if not isinstance(signature, str):
            raise NoneSignatureError
        if len(signature) == 0:
            raise ValueError("Mapper: Signature cannot be empty")

        def get_mapper_cls(other_cls):

            def get_mapper_instance(*args, **kwargs) -> object:
                instance = cls.__container.get(signature)
                if instance is None:
                    instance = other_cls(*args, **kwargs)
                    cls.__container.put(signature, instance)
                return instance

            get_mapper_instance.__decorator__ = "Mapper"
            get_mapper_instance.__decorator_params__ = signature
            return get_mapper_instance

        return get_mapper_cls

    @classmethod
    def Service(cls, signature: str) -> callable:
        if not isinstance(signature, str):
            raise NoneSignatureError
        if len(signature) == 0:
            raise ValueError("Service: Signature cannot be empty")

        def get_service_cls(other_cls):

            def get_service_instance(*args, **kwargs) -> object:
                instance = cls.__container.get(signature)
                if instance is None:
                    instance = other_cls(*args, **kwargs)
                    cls.__container.put(signature, instance)
                return instance

            get_service_instance.__decorator__ = "Service"
            get_service_instance.__decorator_params__ = signature
            return get_service_instance

        return get_service_cls

    @classmethod
    def Controller(cls, path: str) -> callable:
        if not isinstance(path, str):
            raise NoneSignatureError
        if len(path) == 0:
            raise ValueError("Controller: path cannot be empty")

        def get_controller_cls(other_cls):

            def get_controller_instance(*args, **kwargs) -> object:
                instance = cls.__container.get(path)
                if instance is None:
                    instance = other_cls(*args, **kwargs)
                    cls.__container.put(path, instance)
                return instance

            get_controller_instance.__decorator__ = "Controller"
            get_controller_instance.__decorator_params__ = path
            return get_controller_instance

        return get_controller_cls

    @classmethod
    def Resource(cls, names: list[str] | str) -> callable:
        if not isinstance(names, list | str):
            raise ValueError("Resource: the signature of type must be list or str")
        if not all(isinstance(name, str) for name in names):
            raise ValueError("All elements in the list must be strings")
        if len(names) == 0:
            raise ValueError("the names must not be empty")

        def decorator_func(func):
            def wrapper(*args, **kwargs):
                if isinstance(names, str):
                    resources = cls.__container.get(names)
                    if resources is None:
                        raise ValueError(f"the signature {names} does not exist")
                else:
                    resources = kwargs["resources"]
                    for name in names:
                        instance = cls.__container.get(name)
                        if instance is None:
                            raise ValueError(f"the signature {name} does not exist")
                        resources[name] = instance
                func(args[0], resources)

            wrapper.__decorator__ = "Resource"
            return wrapper

        return decorator_func


Component = SpiritApplication.Component
Mapper = SpiritApplication.Mapper
Service = SpiritApplication.Service
Controller = SpiritApplication.Controller
Resource = SpiritApplication.Resource



# @Coding: UTF-8
# @Time: 2024/9/10 13:48
# @Author: xieyang_ls
# @Filename: assemble.py

from typing import TypeVar, Generic

from abc import ABC, abstractmethod

K = TypeVar('K')

V = TypeVar('V')


class Assemble(ABC, Generic[K, V]):

    @abstractmethod
    def put(self, key: K, value: V) -> None:
        pass

    @abstractmethod
    def get(self, key: K) -> V:
        pass

    @abstractmethod
    def remove(self, key: K) -> V:
        pass

    @abstractmethod
    def __len__(self):
        pass

    @abstractmethod
    def clean(self) -> None:
        pass


class HashAssemble(Assemble[K, V], Generic[K, V]):
    __LOAD_FACTOR = 0.75

    __initial_capacity: int = None

    __current_capacity: int = None

    __nodes = None

    __expansion_nodes = None

    def __init__(self, initial_capacity: int = 15) -> None:
        if initial_capacity is None or initial_capacity <= 0:
            self.__initial_capacity = 15
        else:
            self.__initial_capacity = initial_capacity
        self.__current_capacity = 0
        self.__nodes: list[HashAssemble.Node[K, V] | None] = [None] * self.__initial_capacity
        self.__expansion_nodes: None = None

    def __handler(self, flag):
        if flag == 'a':
            self.__current_capacity += 1
        else:
            self.__current_capacity -= 1

    def __setattr__(self, key, value):
        if hasattr(self, key):
            super().__setattr__(key, value)
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{key}'")

    def __getIdCode(self, key: K) -> int:
        try:
            code = hash(key)
        except TypeError:
            code = id(key)
        return abs(code) % self.__initial_capacity

    def __expansion(self):
        self.__initial_capacity *= 2
        self.__expansion_nodes: list[HashAssemble.Node[K, V] | None] = [None] * self.__initial_capacity
        for node in self.__nodes:
            while node is not None:
                idCode: int = self.__getIdCode(node.key)
                if self.__expansion_nodes[idCode] is None:
                    self.__expansion_nodes[idCode] = node
                else:
                    self.__expansion_nodes[idCode].putNodeToLast(node)
                node = node.getNextDifferenceIdNode(idCode, self.__getIdCode)
        self.__nodes = self.__expansion_nodes
        self.__expansion_nodes = None

    def put(self, key: K, value: V) -> None:
        if self.__current_capacity / self.__initial_capacity >= HashAssemble.__LOAD_FACTOR:
            self.__expansion()
        idCode: int = self.__getIdCode(key)
        if self.__nodes[idCode] is None:
            self.__nodes[idCode] = HashAssemble.Node(key, value)
            self.__current_capacity += 1
        elif self.__nodes[idCode].key == key or self.__nodes[idCode].key is key:
            self.__nodes[idCode].value = value
        else:
            self.__nodes[idCode].putNextNode(key, value, self.__handler)

    def get(self, key: K) -> V:
        idCode: int = self.__getIdCode(key)
        if self.__nodes[idCode] is None:
            return None
        elif self.__nodes[idCode].key == key or self.__nodes[idCode].key is key:
            return self.__nodes[idCode].value
        else:
            return self.__nodes[idCode].getNextNode(key)

    def remove(self, key: K) -> V:
        idCode: int = self.__getIdCode(key)
        if self.__nodes[idCode] is None:
            return None
        elif self.__nodes[idCode].key == key or self.__nodes[idCode].key is key:
            value = self.__nodes[idCode].value
            self.__nodes[idCode] = self.__nodes[idCode].nextNode
            self.__current_capacity -= 1
            return value
        else:
            return self.__nodes[idCode].removeNextNode(key, self.__handler)

    def __len__(self):
        return self.__current_capacity

    def clean(self) -> None:
        self.__current_capacity = 0
        self.__initial_capacity = 15
        self.__nodes = [None] * self.__initial_capacity

    class Node(Generic[K, V]):

        key: K = None

        value: V = None

        def __init__(self, key: K, value: V) -> None:
            self.key = key
            self.value = value
            self.nextNode: [HashAssemble.Node | None] = None

        def putNextNode(self, key: K, value: V, handler: callable(str)) -> None:
            if self.nextNode is None:
                self.nextNode = HashAssemble.Node(key, value)
                handler('a')
            elif self.nextNode.key == key or self.nextNode.key is key:
                self.nextNode.value = value
            else:
                return self.nextNode.putNextNode(key, value, handler)

        def getNextNode(self, key: K) -> V:
            if self.nextNode is None:
                return None
            elif self.nextNode.key == key or self.nextNode.key is key:
                return self.nextNode.value
            else:
                return self.nextNode.getNextNode(key)

        def removeNextNode(self, key: K, handler: callable(str)) -> V:
            if self.nextNode is None:
                return None
            elif self.nextNode.key == key or self.nextNode.key is key:
                value = self.nextNode.value
                self.nextNode = self.nextNode.nextNode
                handler('r')
                return value
            else:
                return self.nextNode.removeNextNode(key, handler)

        def putNodeToLast(self, node: Generic[K, V]) -> None:
            if self.nextNode is None:
                self.nextNode = node
            else:
                self.nextNode.putNodeToLast(node)

        def getNextDifferenceIdNode(self, idCode: int, getIdCode: callable(K)) -> Generic[K, V]:
            if self.nextNode is None:
                return None
            differenceIdCode = getIdCode(self.nextNode.key)
            if differenceIdCode == idCode:
                return self.nextNode.getNextDifferenceIdNode(idCode, getIdCode)
            else:
                eNode: HashAssemble.Node[K, V] = self.nextNode
                self.nextNode = None
                return eNode



# @Coding: UTF-8
# @Time: 2024/9/11 0:15
# @Author: xieyang_ls
# @Filename: mysql_handler.py
import datetime

from abc import ABC, abstractmethod

from logging import info, error, INFO, basicConfig

from pyutils_spirit.annotation import singleton

from pymysql import Connect, OperationalError, cursors

from pyutils_spirit.exception import ArgumentException

basicConfig(level=INFO)


class Handler(ABC):

    @abstractmethod
    def select(self, sql: str) -> tuple:
        pass

    @abstractmethod
    def insert(self, sql: str) -> bool:
        pass

    @abstractmethod
    def update(self, sql: str) -> bool:
        pass

    @abstractmethod
    def delete(self, sql: str) -> bool:
        pass

    @abstractmethod
    def manageExecute(self, sql: str) -> bool:
        pass

    @abstractmethod
    def disconnect(self) -> None:
        pass

    @abstractmethod
    def getConnection(self) -> [Connect]:
        pass


@singleton(signature="MysqlHandler")
class MySQLHandler(Handler):
    __cursor = None

    __connection = None

    def __init__(self, args: dict[str, str | int]) -> None:
        try:
            connection: Connect = Connect(
                host=args["host"],
                port=args["port"],
                user=args["user"],
                password=args["password"],
                cursorclass=cursors.DictCursor
            )
            connection.select_db(args["database"])
            info(f"Connected to database {args['database']} successfully!!!")
            info(f"MySQL version: {connection.get_server_info()}")
            cursor = connection.cursor()
            self.__connection = connection
            self.__cursor = cursor
        except (ArgumentException, OperationalError):
            error(f"Connected to database {args['database']} failure")
            raise ArgumentException("please check connected the database arguments")

    def __json_format_list(self, results: list[dict] | tuple[dict]) -> list:
        if isinstance(results, list | tuple):
            for result in results:
                if result is None:
                    continue
                self.__json_format_dict(result)
        return results

    def __json_format_dict(self, result: dict) -> dict:
        for key in result.keys():
            attribute = result[key]
            if attribute is None:
                continue
            if isinstance(attribute, dict):
                result[key] = self.__json_format_dict(attribute)
            elif isinstance(attribute, list):
                result[key] = self.__json_format_list(attribute)
            else:
                if isinstance(attribute, datetime.datetime):
                    result[key] = attribute.isoformat()
        return result

    def select(self, sql: str) -> list | tuple | dict:
        try:
            self.__cursor.execute(sql)
            results = self.__cursor.fetchall()
            return self.__json_format_list(results)
        except OperationalError as e:
            if e.args[0] == 1054:
                return []
            else:
                raise e

    def insert(self, sql: str) -> bool:
        try:
            self.__cursor.execute(sql)
            self.__connection.commit()
            return True
        except Exception as e:
            self.__connection.rollback()
            raise e

    def update(self, sql: str) -> bool:
        try:
            self.__cursor.execute(sql)
            self.__connection.commit()
            return True
        except Exception as e:
            self.__connection.rollback()
            raise e

    def delete(self, sql: str) -> bool:
        try:
            self.__cursor.execute(sql)
            self.__connection.commit()
            return True
        except Exception as e:
            self.__connection.rollback()
            raise e

    def manageExecute(self, sql: str) -> bool:
        try:
            self.__cursor.execute(sql)
            self.__connection.commit()
            return True
        except Exception as e:
            self.__connection.rollback()
            raise e

    def getConnection(self) -> [Connect]:
        return self.__connection, self.__cursor

    def disconnect(self) -> None:
        self.__connection.close()



# @Coding: UTF-8
# @Time: 2024/9/25 15:58
# @Author: xieyang_ls
# @Filename: multipart_file.py

import os

import shutil

import tempfile

from typing import BinaryIO


class MultipartFile(object):
    __temp_file_handler__: tempfile.TemporaryFile = None

    __temp_file_path__: str | None = None

    __temp_file__: BinaryIO | None = None

    __form_data__: dict | None = None

    def __init__(self, **kwargs):
        self.__form_data__ = kwargs
        self.__temp_file_handler__ = kwargs.get("file")
        try:
            self.__temp_file_path__ = tempfile.mktemp()  # 创建临时文件路径
            self.__temp_file__ = open(self.__temp_file_path__, 'wb')
            # 将上传的文件写入临时文件
            shutil.copyfileobj(self.__temp_file_handler__, self.__temp_file__, length=20 * 1024 * 1024)
        except Exception as e:
            raise Exception(e)
        finally:
            if self.__temp_file__ is not None:
                self.__temp_file__.close()  # 确保文件关闭

    def copy(self, file_path: str):
        try:
            shutil.copyfile(self.__temp_file_path__, file_path)
        except Exception as e:
            raise Exception(e)

    def get(self, key: str) -> object:
        return self.__form_data__.get(key)

    def close(self):
        if self.__temp_file_handler__ is not None:
            self.__temp_file_handler__.close()
            self.__temp_file_handler__ = None
        if self.__temp_file__ is not None:
            self.__temp_file__.close()
            self.__temp_file__ = None
        if self.__temp_file_path__ is not None and os.path.exists(self.__temp_file_path__):
            os.remove(self.__temp_file_path__)  # 确保临时文件删除
            self.__temp_file_path__ = None

    def __str__(self):
        return self.__form_data__.__str__()
